--[[
Name: BankAccountant
Revision: $Rev: 6 $
Developed by: Choumarin
Documentation: http://www.wowace.com/wiki/BankAccountant
SVN: http://svn.wowace.com/wowace/trunk/BankAccountant
Description: Keep counts of deposits and withdraws to your guild bank caracter.
Todo: (prior desc)
    - error catching for improper cmd inputs (price / add / remove)
    - caracter management (not mixing banks)
    - GUI
    - manage alts as one player
    - who == "all" issue, don't think many players uses this name, but if there is ...
    - export to web/board
]]



BankAccountant = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1", "FuBarPlugin-2.0")
local abacus = AceLibrary("Abacus-2.0")
local tablet   = AceLibrary('Tablet-2.0')
local L = AceLibrary("AceLocale-2.2"):new("BankAccountant")
BankAccountant.gwho = ""
BankAccountant.queue = {}

BankAccountant.hasIcon = "Interface\\Icons\\INV_Jewelry_Ring_03"
BankAccountant.cannotDetachTooltip = true


local dewdrop = AceLibrary("Dewdrop-2.0")

local options = { 
    type='group',
    args = {
        show = {
            type = "text",
            name = L["Show"],
            desc = L["Show transactions"],
            get = false,
            set = "ShowTransaction",
            usage = L["<who|all>"],
        },
        setprice = {
            type = "text",
            name = L["Set Price"],
            desc = L["Sets an item price."],
            get = false,
            set = "SetPriceCmd",
            usage = L["<itemLink> <price>"],
        },
        additem = {
            type = "text",
            name = L["Add Item"],
            desc = L["Manually adds an item."],
            get = false,
            set = "addItemCmd",
            usage = L["<withdraw|donate> <who> <itemLink> [number]"],
        },
        addmoney = {
            type = "text",
            name = L["Add Money"],
            desc = L["Manually adds money."],
            get = false,
            set = "addMoneyCmd",
            usage = L["<withdraw|donate> <who> <amount>"],
        },
        removeitem = {
            type = "text",
            name = L["Remove Item"],
            desc = L["Manually removes an item."],
            get = false,
            set = "removeItemCmd",
            usage = L["<withdraw|donate> <who> <itemLink> [number]"],
        },
        removemoney = {
            type = "text",
            name = L["Remove Money"],
            desc = L["Manually removes money."],
            get = false,
            set = "removeMoneyCmd",
            usage = L["<withdraw|donate> <who> <amount>"],
        },
        balance = {
            type = "text",
            name = L["Balance"],
            desc = L["Shows the balance of someone."],
            get = false,
            set = "PrintBalance",
            usage = L["<who|all>"],
        },
        debug = {
            type = 'toggle',
            name = L["Debug"],
            desc = L["Display debug information."],
            get = "IsDebug",
            set = "ToggleDebug",
        },
        welcome = {
            type = 'toggle',
            name = L["Welcome"],
            desc = L["Display welcome message."],
            get = "IsWelcome",
            set = "ToggleWelcome",
        },
        reset = {
            type = 'execute',
            name = L["Reset"],
            desc = L["Reset the whole BankAccountant DB, erasing all stored information."],
            func =  function()
                        BankAccountant:ResetDB("profile")
                    end,
            confirm = L["Are you sure you want to delete all BankAccountant informations ?"],
        },
        gui = {
            type = 'execute',
            name = L["GUI"],
            desc = L["Toggles the GUI."],
            func = "ToggleGUI"
        }
    },
}

BankAccountant.OnMenuRequest = options

function BankAccountant:OnClick()
    self:ToggleGUI()
end

BankAccountant:RegisterChatCommand({"/bankaccountant", "/ba"}, options)

BankAccountant:RegisterDB("BankAccountantDB", "BankAccoutantDBPC")
BankAccountant:RegisterDefaults("profile", {
    debug = false,
    welcome = true,
    prices = {},
    names = {},
})


function BankAccountant:OnInitialize()
    -- Called when the addon is loaded
end

function BankAccountant:OnDisable()
    -- Called when the addon is disabled
    -- unhook functions or automatic ?
end

function BankAccountant:OnEnable()
    -- Called when the addon is activated
    if self.db.profile.welcome then
        self:Print(L["Hello Bank Manager !"])
    end
    self:Hook("TakeInboxItem", true);
    self:Hook("TakeInboxMoney", true);
    self:Hook("SendMail", true);
    self:RegisterEvent("MAIL_SHOW")
    self:RegisterEvent("MAIL_CLOSED")

end

function BankAccountant:Debug(text)
    if self.db.profile.debug then
        self:Print(text)
    end
end

function BankAccountant:ToggleDebug()
    self.db.profile.debug = not self.db.profile.debug
end

function BankAccountant:IsDebug()
    return self.db.profile.debug
end

function BankAccountant:ToggleWelcome()
    self.db.profile.welcome = not self.db.profile.welcome
end

function BankAccountant:IsWelcome()
    return self.db.profile.welome
end

function BankAccountant:IsInTable(table, item)
    for _, foo in pairs(table) do
        if foo == item then
            return true
        end
    end
    return false
end


-- Hooks

function BankAccountant:TakeInboxItem(index)
    local _, _, sender = GetInboxHeaderInfo(index)
    local _, _, count = GetInboxItem(index)
    
    self:Debug(sender .. " donates " .. GetInboxItemLink(index) .. "x" .. count)
    tinsert(self.queue, {["action"] = "AddItem", ["way"] = "donate", sender, GetInboxItemLink(index), count})
        
    return self.hooks.TakeInboxItem(index)
end

function BankAccountant:TakeInboxMoney(index)
    local _, _, sender, _, money = GetInboxHeaderInfo(index)

    self:Debug(sender .. " donates " .. abacus:FormatMoneyFull(money, true) )
    tinsert(self.queue, {["action"] = "AddMoney", ["way"] = "donate", sender, money})    
    
    return self.hooks.TakeInboxMoney(index)
end

function BankAccountant:SendMail(recipient, subject, body) -- do not input right now, wait for event
    money = GetSendMailMoney()
    local itemName, texture, count = GetSendMailItem()
    if itemName ~= nil then
        local _, itemLink = GetItemInfo(itemName)
        --self:Debug("sending " .. recipient .. " " .. itemLink .. "x" .. count)
        tinsert(self.queue, {["action"] = "AddItem", ["way"] = "withdraw", recipient, itemLink, count})
    
    end 

    if money ~= 0 then
        self:Debug("sending " .. recipient .. " : " .. abacus:FormatMoneyFull(money, true))
        tinsert(self.queue, {["action"] = "AddMoney", ["way"] = "withdraw", recipient, money})
            
    end
    
    return self.hooks.SendMail(recipient, subject, body)
end

-- Events

function BankAccountant:MAIL_SHOW()
	if (not self:IsEventRegistered("MAIL_FAILED")) then self:RegisterEvent("MAIL_FAILED") end
	if (not self:IsEventRegistered("MAIL_SEND_SUCCESS")) then self:RegisterEvent("MAIL_SEND_SUCCESS") end
	if (not self:IsEventRegistered("BAG_UPDATE")) then self:RegisterEvent("BAG_UPDATE") end
	if (not self:IsEventRegistered("PLAYER_MONEY")) then self:RegisterEvent("PLAYER_MONEY") end
	if (not self:IsEventRegistered("UI_ERROR_MESSAGE")) then self:RegisterEvent("UI_ERROR_MESSAGE") end
end

function BankAccountant:MAIL_CLOSED()
    if (self:IsEventRegistered("MAIL_FAILED")) then self:UnregisterEvent("MAIL_FAILED") end
    if (self:IsEventRegistered("MAIL_SEND_SUCCESS")) then self:UnregisterEvent("MAIL_SEND_SUCCESS") end
    if (self:IsEventRegistered("BAG_UPDATE")) then self:UnregisterEvent("BAG_UPDATE") end
    if (self:IsEventRegistered("PLAYER_MONEY")) then self:UnregisterEvent("PLAYER_MONEY") end
    if (self:IsEventRegistered("UI_ERROR_MESSAGE")) then self:UnregisterEvent("UI_ERROR_MESSAGE") end
end

function BankAccountant:MAIL_FAILED() -- fires when retreiving mail !?!?
    -- clear queue
    self:Debug("MAIL_FAILED")
    if  self.queue[1] ~= nil then
        if self.queue[1]["way"] == "withdraw" then -- last mail was sending ? if yes then delete queue
            self.queue = {}
        end
    end
end

function BankAccountant:UI_ERROR_MESSAGE()
    self:Debug("UI_ERROR_MESSAGE : " .. tostring(arg1))
    if arg1 == ERR_INV_FULL or arg1 == ERR_ITEM_MAX_COUNT then
        if  self.queue[1] ~= nil then
            if self.queue[1]["way"] == "donate" then
                self.queue = {}
            end
        end
    end
end

function BankAccountant:MAIL_SEND_SUCCESS()
    -- confirm input
    self:Debug("MAIL_SEND_SUCCESS")
    if  self.queue[1] ~= nil then
        self:ProcessQueue()
    end
end

function BankAccountant:BAG_UPDATE()
    -- confirm input
    self:Debug("BAG_UPDATE")
    if  self.queue[1] ~= nil then
        self:ProcessQueue()
    end
end


function BankAccountant:PLAYER_MONEY()
    -- confirm input
    self:Debug("PLAYER_MONEY")
    if  self.queue[1] ~= nil then
        self:ProcessQueue()    
    end
end

function BankAccountant:ProcessQueue()
   self:Debug(self.queue[1]["action"] .. self.queue[1]["way"])
    for i, queueItem in pairs(self.queue) do 
        if queueItem["action"] == "AddItem" then
            self:AddItem(queueItem["way"], queueItem[1], queueItem[2], queueItem[3])
        elseif queueItem["action"] == "AddMoney" then
            self:AddMoney(queueItem["way"], queueItem[1], queueItem[2])
        end
    end
    self.queue = {}
end

function BankAccountant:TidyName(who)
    if self.db.profile.names[who][itemId]["donate"] <=0 then
        self.db.profile.names[who][itemId]["donate"] = 0
    end
    if self.db.profile.names[who][itemId]["withdraw"] <=0 then
        self.db.profile.names[who][itemId]["withdraw"] = 0
    end
    if self.db.profile.names[who][itemId]["withdraw"] <=0 and self.db.profile.names[who][itemId]["donate"] <= 0 then
        self.db.profile.names[who][itemId] = nil
    end
    if self.db.profile.names[who]["money"]["donate"] <=0 then
        self.db.profile.names[who]["money"]["donate"] = 0
    end
    if self.db.profile.names[who]["money"]["withdraw"] <=0 then
        self.db.profile.names[who]["money"]["withdraw"] = 0
    end
    if self.db.profile.names[who] == {
        ["money"] = {
            ["donate"] = 0,
            ["withdraw"] = 0,
        },
    } then
        self.db.profile.names[who] = nil
    end
end

function BankAccountant:AddItem(way, who, itemLink, count)
    count = tonumber(count)
	local _,_,itemId = string.find(itemLink,"item:(%d+):(%d+):(%d+):%d+");
    self:Debug("adding " .. who .. " " .. itemLink .. "( " .. itemId .. " )" .. "x" .. count)
    if self.db.profile.names[who] == nil then
        self.db.profile.names[who] = {
            ["money"] = {
                ["donate"] = 0,
                ["withdraw"] = 0,
            },
        }
    end
    if self.db.profile.names[who][itemId] == nil then
        self.db.profile.names[who][itemId] = {
            ["donate"] = 0,
            ["withdraw"] = 0,
        }
    end
    self.db.profile.names[who][itemId][way] = self.db.profile.names[who][itemId][way] + count
    end

function BankAccountant:AddMoney(way, who, money)
    money = tonumber(money)
    self:Debug("adding " .. who .. " " .. abacus:FormatMoneyFull(money, true))
    if self.db.profile.names[who] == nil then
        self.db.profile.names[who] = {
            ["money"] = {
                ["donate"] = 0,
                ["withdraw"] = 0,
            },
        }
    end
    self.db.profile.names[who]["money"][way] = self.db.profile.names[who]["money"][way] + money
end

function BankAccountant:RemoveItem(way, who, itemLink, count)
    count = tonumber(count)
	local _,_,itemId = string.find(itemLink,"item:(%d+):(%d+):(%d+):%d+");
    self:Debug("removing " .. who .. " " .. itemLink .. "( " .. itemId .. " )" .. "x" .. count)
    if names[who][itemId] ~= nil then
        names[who][itemId][way] = names[who][itemId][way] - count
    end
    self:TidyName(who)
end

function BankAccountant:RemoveMoney(way, who, money)
    money = tonumber(money)
    self:Debug("removing " .. who .. " " .. abacus:FormatMoneyFull(money, true))
	if names[who]["money"] ~= nil then
        names[who]["money"][way] = names[who]["money"][way] - money 
	end
    self:TidyName(who)
end

function BankAccountant:ShowTransaction(who)
    if who == "all" then
        for who, data in pairs(self.db.profile.names) do
            self:Print(who .. " :")
            for what, data2 in pairs(data) do
                if what == "money" then
                    self:Print("   " .. L["Money"] .. " : ")
                    self:Print("      " .. L["Donations :"] .. abacus:FormatMoneyFull(data2["donate"], true))
                    self:Print("      " .. L["Withdrawals :"] .. abacus:FormatMoneyFull(data2["withdraw"], true))
                else
                    _, itemLink = GetItemInfo(what)
                    if self.db.profile.prices[what] == nil then self.db.profile.prices[what] = 0 end
                    self:Print("   " .. itemLink .. " :" )
                    self:Print("      " .. L["Donations :"] .. data2["donate"] .. " (" .. self.db.profile.prices[what] * data2["donate"] .. ")")
                    self:Print("      " .. L["Withdrawals :"] .. data2["withdraw"] .. " (" .. self.db.profile.prices[what] * data2["withdraw"] .. ")")
                end
            end
            self:Print("")
        end
    elseif self.db.names[who] ~= nil then
        self:Print(who .. " :")
        for what, data2 in pairs(self.db.names[who]) do
            if what == "money" then
                self:Print("   " .. L["Money"] .. " : ")
                self:Print("      " .. L["Donations :"] .. abacus:FormatMoneyFull(data2["donate"], true))
                self:Print("      " .. L["Withdrawals :"] .. abacus:FormatMoneyFull(data2["withdraw"], true))
            else
                _, itemLink = GetItemInfo(what)
                if self.db.profile.prices[what] == nil then self.db.profile.prices[what] = 0 end
                self:Print("   " .. itemLink .. " :" )
                self:Print("      " .. L["Donations :"] .. data2["donate"] .. " (" .. self.db.profile.prices[what] * data2["donate"] .. ")")
                self:Print("      " .. L["Withdrawals :"] .. data2["withdraw"] .. " (" .. self.db.profile.prices[what] * data2["withdraw"] .. ")")
            end
        end
    else
        self:Print(L["No data for this player."])
    end
end

function BankAccountant:SetPriceCmd(cmd)
    local _, _, itemLink, price  = string.find(cmd, "(|c.+|h|r)%s(%d+)")
    self:SetPrice(itemLink, price)
end

function BankAccountant:addItemCmd(cmd)
    local _, _, way, who, itemLink, count  = string.find(cmd, "(%a+)%s(%a+)%s(|c.+|h|r)%s?(%d*)")
    if count == nil then
        count = 1
    end
    self:Debug(way .. who .. itemLink .. ' (' .. ') ' .. count)
    self:AddItem(way, who, itemLink, count)
end

function BankAccountant:addMoneyCmd(cmd)
    local _, _, way, who, money  = string.find(cmd, "(%a+)%s(%a+)%s(%d+)")
    self:Debug(way .. who .. money)
    self:AddMoney(way, who, money)
end

function BankAccountant:removeItemCmd(cmd)
    local _, _, way, who, itemLink, count  = string.find(cmd, "(%a+)%s(%a+)%s(|c.+|h|r)%s(%d+)")
    self:Debug(way .. who .. itemLink .. ' (' .. ') ' .. count)
    self:RemoveItem(way, who, itemLink, count)
end

function BankAccountant:removeMoneyCmd(cmd)
    local _, _, way, who, money  = string.find(cmd, "(%a+)%s(%a+)%s(%d+)")
    self:Debug(way .. who .. money)
    self:RemoveMoney(way, who, money)
end


function BankAccountant:SetPrice(itemLink, price)
    local _, _, itemId = string.find(itemLink, "item:(%d+):(%d+):(%d+):%d+");
    price = tonumber(price)
    self:Print(itemLink .. L[" is now worth "] .. price)
    self.db.profile.prices[itemId] = price
end

function BankAccountant:GetPrice(way, who)
    local total = 0
    for what, data in pairs(self.db.profile.names[who]) do
        if what ~= "money" then
            if self.db.profile.prices[what] == nil then self.db.profile.prices[what] = 0 end
            total = total + self.db.profile.prices[what] * data[way] 
        end
    end
    return total
end

function BankAccountant:GetBalance(who)
    if self.db.profile.names[who] ~= nil then
        return self:GetPrice("donate", who) - self:GetPrice("withdraw", who)
    end
    return 0
end

function BankAccountant:PrintBalance(who)
    if who == "all" then
        for who in pairs(self.db.profile.names) do
            self:Print(who .. " : " .. self:GetBalance(who) .. " ( +" .. self:GetPrice("donate", who) .. " -" .. self:GetPrice("withdraw", who) .. " )")
        end
    elseif self.db.profile.names[who] ~= nil then
            self:Print(who .. " : " .. self:GetBalance(who) .. " ( +" .. self:GetPrice("donate", who) .. " -" .. self:GetPrice("withdraw", who) .. " )")
    else
        self:Print(L["No data for this player."])
    end
end

function BankAccountant:ToggleGUI()
    if not tablet:IsRegistered("Charlist") or not tablet:IsRegistered("ObjList") then
        self:InitGUI()
        tablet:Open('Charlist')				
        tablet:Open('ObjList')
--        tablet:Open(Minimap)--fixme
        BankAccountant.IsOpened = true
    else
        if BankAccountant.IsOpened then
            tablet:Open('Charlist')				
            tablet:Open('ObjList')
--            tablet:Open(Minimap)--fixme
            BankAccountant.IsOpened = not BankAccountant.IsOpened
        else
            tablet:Close('Charlist')				
            tablet:Close('ObjList')
            BankAccountant.IsOpened = not BankAccountant.IsOpened
        end
    end
end

function BankAccountant:InitGUI()

    tablet:Register('Charlist', 'strata', "HIGH",
            'point', function()
                return "TOPRIGHT", "TOPLEFT"
            end,
            'minWidth', 200, 'maxHeight', 200,
			'cantAttach', true, 'dontHook', true, 'showTitleWhenDetached', true,  
            'children', function()
                tablet:SetTitle("BankAccountant")
				local cat = tablet:AddCategory('columns', 2, 'text', L["Character"], 'text2', L["Balance"],
					'showWithoutChildren', true, 'child_indentation', 5)
				
				for who, data in pairs(self.db.profile.names) do               
				    cat:AddLine(
                        'text', who, 
                        'text2', self:GetBalance(who),
                        'func', function()
                            BankAccountant.gwho = who
                            tablet:Refresh('ObjList')
                        end
                    )
				end
				
			end)
				


    tablet:Register('ObjList', 'strata', "HIGH",
            'point', function()
                return "TOPRIGHT", "TOPLEFT"
            end,
            'minWidth', 200, 'maxHeight', 200,
			'cantAttach', true, 'dontHook', true, 'showTitleWhenDetached', true,  
            'children', function()
                tablet:SetTitle(BankAccountant.gwho)
				local cat = tablet:AddCategory('columns', 4, 'text', L["Object"], 'text2', L["Price"], 'text3', L["Donated"], 'text4', L["Withdrawed"],
					'showWithoutChildren', true, 'child_indentation', 5)
				
				if self.db.profile.names[BankAccountant.gwho] ~= nil then
    				local data = self.db.profile.names[BankAccountant.gwho]
                    for what, data2 in pairs(data) do
                        if what == "money" then
                            itemLink = L["Money"]
                            d = abacus:FormatMoneyFull(data2["donate"], true)
                            w = abacus:FormatMoneyFull(data2["withdraw"], true)
                        elseif what ~= "money" then
                            _, itemLink = GetItemInfo(what)
                            d = data2["donate"]
                            w = data2["withdraw"]
                        end		
                        cat:AddLine(
                            'text', itemLink, 
                            'text2', self.db.profile.prices[what],
                            'text3', d, 
                            'text4', w
                        )
                    end
                else
                    cat:AddLine('text', "Nothing to show")
                end	
                
			end)

end


--[[]

tablet:Register(Minimap,
        'point', function()
            return "TOPRIGHT", "TOPLEFT"
        end,
        'dontHook', true,
        'strata', "TOOLTIP",
        'frameLevel', 1,
        'clickable', true,
        'children', function()
			local cat = tablet:AddCategory()
            cat:AddLine('text', "TEST")				
		end)
tablet:Open(Minimap)

--]]
